/*
   Licensed Materials - Property of IBM
   5725-G92 (C) Copyright IBM Corp. 2011, 2012. All Rights Reserved.
   US Government Users Restricted Rights - Use, duplication or
   disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
 */

-- create db <your-db-name> using codeset 'UTF-8' territory US pagesize 32768

-- -------------------------------------------------
-- Clean resources first
-- -------------------------------------------------
-- drop table puremeap.feedback;
-- drop table puremeap.installed_apps;
-- drop table puremeap.application_storage;
-- drop table puremeap.application_storage_ref;
-- drop table puremeap.application_policies;
-- drop table puremeap.devices;
-- drop table puremeap.application_md;
-- drop table puremeap.acl_groups;
-- drop table puremeap.acl_resources;
-- drop table puremeap.acl_contents;
-- drop table puremeap.principals;

-- drop trigger puremeap.insertPrincipalTrigger;
-- drop trigger puremeap.deletePrincipalTrigger;
-- drop procedure puremeap.insertPrincipalProc;
-- drop procedure puremeap.deletePrincipalProc;


-- -------------------------------------------------
-- A Security Principal
-- -------------------------------------------------
create table puremeap.principals
    -- Database ID, an identity field, hidden from users, automatically generated
(id integer not null generated always as identity (start with 1, increment by 1),
    -- Principal name
 name varchar(128) not null,
    -- Principal full name
 full_name varchar(256) not null, 
    -- Principal type, 'USER', 'GROUP', etc.
 type varchar(8) not null,
 constraint principal_id primary key (id),
 constraint principal_name unique(name))
; 

-- -------------------------------------------------
-- Principal mapping to support groups
-- -------------------------------------------------
create table puremeap.acl_groups
    -- Pointer to group (a principal)
(group_fk integer not null,
    -- Pointer to principal
 principal_fk integer not null,
    -- Both columns comprise the primary key
 primary key (group_fk, principal_fk),
 constraint aclgrp_to_grp foreign key (group_fk) references puremeap.principals(id) on delete cascade,
 constraint aclgrp_to_pcp foreign key (principal_fk) references puremeap.principals(id) on delete cascade)
;

-- -------------------------------------------------
-- Resource Access Control
-- Resources are defined in ResourceAccessControl
-- -------------------------------------------------
create table puremeap.acl_resources
    -- Database ID, an identity field, hidden from users, automatically generated
(id integer not null generated always as identity (start with 1, increment by 1),
    -- Pointer to principal
 principal_fk integer not null,
    -- Resource identifier
 resource smallint not null,
    -- Access level
 access smallint not null, 
 constraint aclres_id primary key (id),
 constraint aclres_to_pcp foreign key (principal_fk) references puremeap.principals(id) on delete cascade)
; 

-- -------------------------------------------------
-- Tracking activated devices
-- -------------------------------------------------
create table puremeap.devices
    -- Database ID, an identity field, hidden from users
(id integer generated by default as identity,
    -- When was this device added to the database
 created timestamp not null,
    -- When was the last time the device was updated
 updated timestamp not null,
    -- The ID of the user that activated the device
 user_id varchar(128) not null,
    -- A UUID that identify the activated device. On iOS the actual device 
    -- UUID, on Android UUIDs and MAC addresses are not reliable, we generate
    -- and record our own UUIDs 
 device_id varchar(256) not null,
    -- One of Android, or iOS
 os varchar(32) not null,
    -- Apple, Samsung ...
 maker varchar(256) not null,
    -- iPad, iPhone, Nexus S, ...
 model varchar(32) not null,
    -- Full OS String, e.g., iOS/4.3.2 
 os_description varchar(256),
    -- User visible name, e.g., Gals' Android  
 nickname varchar(64),
 active char(1) not null default 'Y',
 constraint devices_id primary key (id),
 constraint unique_dev_id unique (device_id));

-- -------------------------------------------------
-- Applications in the repository.
-- Allows users to search for apps. Much of the data
-- is extracted from the application file when 
-- uploaded into the repository
-- -------------------------------------------------
create table puremeap.application_md
    -- Database ID, an identity field, hidden from users
(id integer generated by default as identity,
    -- When was the application added to the repository
 created timestamp not null,
    -- When was the last time the application was updated
 updated timestamp not null,
    -- When was the last time the application was updated 
 mandatory char(1) not null default 'N',
    -- Author name 
 author varchar(128) not null default 'guest',
    -- Package ID, must be unique  
 pkg varchar(256) not null,
    -- Package ID (numeric), must be unique
 pkg_id integer not null,
    -- App version, e.g., 1.0 , internal version
 version varchar(16) not null,
    -- App version name, e.g., 1.0, this is the commercial version presented on the mobile devices 
 version_name varchar(256),
    -- App version as a number, it's handy for making comparisons, or calculating a maximum  
 version_num double not null,
    -- Short, user visible name  
 label varchar(256) not null,
    -- Short, user visible description 
 description varchar(2048),
    -- What packaging are we using, Android (APK), iOS (IPA) or HTML5 (zip) 
 os varchar(32) not null,
    -- Type of the application's primary resource, 
    -- namely 'apk' for Android, 'ipa' for iOS, 'zip' for HTML5
    -- It's redundant information, but it's very handy to check whether an application record is actually associated 
    -- to a resource of the right type 
 resource_type varchar(32) not null,    
    -- Apple's APNs Certificate password - Relevant to iOS applications only, otherwise it should be an empty string
 apns_certificate_password varchar(32) not null,      
    -- APNs Gateway, can be 'DEVELOPMENT', 'PRODUCTION', or an empty string
 apns_gateway varchar(16) not null,
 	-- Is this an installer?
 is_installer char(1) not null default 'N',
    -- Can this application be installed by users
 active char(1) not null default 'Y',
 constraint apmd_id primary key (id),
 constraint unique_app unique (pkg, version));

-- -------------------------------------------------
-- A Reference to an application resource (icon, 
-- application package, profile etc.)
-- -------------------------------------------------
create table puremeap.application_storage_ref
    -- Database ID, an identity field, hidden from users
(id integer generated by default as identity,
    -- Pointer to the application record
 apmd_fk integer not null,
    -- The type of resource (zip, icon, ipa, apk ...)
 type varchar(64) not null,
    -- Size in bytes
 file_size integer not null,
    -- Was this resources derived from the content of the application package
 is_derived char(1) not null default 'N',
 	-- Remote URL for an application file stored elsewhere
 remote_url varchar(1024) not null default 'NONE',
 	-- The filename of the resource when it has been uploaded
 filename varchar(1024),
 
 constraint app_storage_ref_id primary key (id), 
 constraint stor_ref_to_apmd foreign key (apmd_fk) references puremeap.application_md(id) on delete cascade);

-- -------------------------------------------------
-- A BLOB containing an application resource (icon, 
-- application package, profile etc.)
-- -------------------------------------------------
create table puremeap.application_storage
    -- Database ID, an identity field, hidden from users
(id integer generated by default as identity, 
    -- Pointer to the reference
 ap_fk integer,
    -- The actual resource content
 content blob (256M) not null,
 constraint app_storage_id primary key (id),
 constraint stor_to_stor_ref foreign key (ap_fk) references puremeap.application_storage_ref(id) on delete cascade);

-- -------------------------------------------------
-- A record detailing an installation on a specific  
-- device
-- -------------------------------------------------
create table puremeap.installed_apps
    -- Database ID, an identity field, hidden from users
(id integer generated by default as identity,
    -- When was this entry created
 created timestamp not null,
     -- Last time this entry was updated
 updated timestamp not null,
     -- Pointer to the device
 device_fk integer not null,
     -- Pointer to the application
 apmd_fk integer not null,
     -- Should the application be updated when a new version entered 
 to_update char(1) not null default 'Y',
     -- Installation status
 status varchar(32) not null,
     -- Registration data
 registration_data clob(32768) not null,
 constraint installed_apps_id primary key (id),
 constraint unique_devapp_pair unique (device_fk, apmd_fk),
 constraint inst_app_to_device foreign key (device_fk) references puremeap.devices(id),
 constraint inst_app_to_app foreign key (apmd_fk) references puremeap.application_md(id));

-- -------------------------------------------------
-- A Management Policy
-- -------------------------------------------------
create table puremeap.application_policies
    -- Database ID, an identity field, hidden from users
(id integer generated by default as identity,
    -- When was this policy added to the database
 created timestamp not null,
 	-- A unique ID that describes this policy
 policy_id varchar(256) not null,
    -- User-visible name  
 policy_name varchar(256) not null,
 	-- true if and only if this policy should be executed by each client only once
 one_time char(1) not null,
    -- A Pointer to the application
 apmd_fk integer,
     -- Pointer to the device, will be null if this policy is not device-specific
 device_fk integer,
 	-- true if and only if the application should be blocked
 block char(1) not null,
 	-- A message to be displayed to the user (can be empty)
 message varchar(256),
 	-- An operation to be executed. Either UNINSTALL, WIPE, UPDATE or UPDATE_HYBRID (can be empty) 
 operation varchar(32),
 constraint app_policies_id primary key (id),
 constraint unique_app_pol_id unique (policy_id),
 constraint app_pol_to_app foreign key (apmd_fk) references puremeap.application_md(id) on delete cascade);
  
 -- not supported by derby 10.5.3 create sequence puremeap.policy_id start with 0;
 
-- -------------------------------------------------
-- A Feedback item
-- -------------------------------------------------

create table puremeap.feedback
    -- Database ID, an identity field, hidden from users
(id integer generated by default as identity,
    -- When was this entry created
 created timestamp not null,
      -- Pointer to the application
 apmd_fk integer not null,
     -- Pointer to the device
 device_fk integer not null,
  
 feedback_comment varchar(2048),
 
     -- Rating (numerical 1-10)
 rating integer,
 
 constraint feedback_id primary key (id),
 constraint feedback_to_device foreign key (device_fk) references puremeap.devices(id),
 constraint feedback_to_app foreign key (apmd_fk) references puremeap.application_md(id))
 ;

-- -------------------------------------------------
-- Principal mapping to support content ACL
-- -------------------------------------------------
create table puremeap.acl_contents
    -- DatabaseID, an identity field, hidden from users, automatically generated
(table_id integer not null,
    -- Reference to content identifier: the content id on the corresponding table
 content_id integer not null,
    -- Pointer to principal with access privileges for this content
 principal_fk integer not null,
    -- Create access
 can_create smallint not null,
    -- List access
 can_list smallint not null,
    -- Update access
 can_update smallint not null,
    -- Delete access
 can_delete smallint not null,
 constraint aclcont_id primary key (table_id,content_id,principal_fk),
 constraint aclcont_to_pcp foreign key (principal_fk) references puremeap.principals(id) on delete cascade)
;

-- -------------------------------------------------
-- Create special principals (groups)
-- -------------------------------------------------
insert into puremeap.principals (name,full_name,type) values ('Administrators','Group of administrators','GROUP');
insert into puremeap.acl_groups (group_fk,principal_fk) values (1,1);
insert into puremeap.principals (name,full_name,type) values ('Everyone','Group of all registered users','GROUP');
insert into puremeap.acl_groups (group_fk,principal_fk) values (2,2);
insert into puremeap.principals (name,full_name,type) values ('Nobody','Group of pending deleted users','GROUP');
insert into puremeap.acl_groups (group_fk,principal_fk) values (3,3);

-- -------------------------------------------------
-- Create special principals (users)
-- Manually insert users into groups
-- -------------------------------------------------
insert into puremeap.principals (name,full_name,type) values ('appcenteradmin','Administrator','USER');
insert into puremeap.acl_groups (group_fk,principal_fk) values (1,4);
insert into puremeap.acl_groups (group_fk,principal_fk) values (2,4);
insert into puremeap.acl_groups (group_fk,principal_fk) values (4,4);
insert into puremeap.principals (name,full_name,type) values ('demo','Demo User','USER');
insert into puremeap.acl_groups (group_fk,principal_fk) values (2,5);
insert into puremeap.acl_groups (group_fk,principal_fk) values (5,5);
insert into puremeap.principals (name,full_name,type) values ('guest','Guest User','USER');
insert into puremeap.acl_groups (group_fk,principal_fk) values (2,6);
insert into puremeap.acl_groups (group_fk,principal_fk) values (6,6);

-- -------------------------------------------------
-- Create resource access (all users are admins for now)
-- -------------------------------------------------
insert into puremeap.acl_resources (principal_fk,resource,access) values (1,0,15);
insert into puremeap.acl_resources (principal_fk,resource,access) values (1,1,15);
insert into puremeap.acl_resources (principal_fk,resource,access) values (1,2,15);
insert into puremeap.acl_resources (principal_fk,resource,access) values (1,3,15);
insert into puremeap.acl_resources (principal_fk,resource,access) values (1,4,15);
insert into puremeap.acl_resources (principal_fk,resource,access) values (1,5,15);

insert into puremeap.acl_resources (principal_fk,resource,access) values (5,0,15);
insert into puremeap.acl_resources (principal_fk,resource,access) values (5,1,15);
insert into puremeap.acl_resources (principal_fk,resource,access) values (5,2,15);
insert into puremeap.acl_resources (principal_fk,resource,access) values (5,3,15);
insert into puremeap.acl_resources (principal_fk,resource,access) values (5,4,15);
insert into puremeap.acl_resources (principal_fk,resource,access) values (5,5,15);

insert into puremeap.acl_resources (principal_fk,resource,access) values (6,0,15);
insert into puremeap.acl_resources (principal_fk,resource,access) values (6,1,15);
insert into puremeap.acl_resources (principal_fk,resource,access) values (6,2,15);
insert into puremeap.acl_resources (principal_fk,resource,access) values (6,3,15);
insert into puremeap.acl_resources (principal_fk,resource,access) values (6,4,15);
insert into puremeap.acl_resources (principal_fk,resource,access) values (6,5,15);


